@@ -41,3 +41,10 @@
  */
 
 #define	DNS_TRYAGAIN
+
+/*
+ * set the default umask.
+ */
+
+#define CMASK 022
+
