@@ -100,7 +100,7 @@ parsetime(char *whattime)
     while (match && isalpha(*whattime) && isupper(*whattime)) {
         match = 0;
         for (loop = 0; loop < 8; loop++) {
-            if (strncmp(days[loop], whattime, 2) == NULL) {
+            if (strncmp(days[loop], whattime, 2) == 0) {
                 whattime += 2;
                 match = 1;
                 if ((wday == loop) | ((loop == 7) && wday && (wday < 6))) {
@@ -110,7 +110,7 @@ parsetime(char *whattime)
         }
     }
 
-    if (strncmp(whattime, "Any", 3) == NULL) {
+    if (strncmp(whattime, "Any", 3) == 0) {
         validday = 1;
         whattime += 3;
     }
@@ -428,7 +428,7 @@ acl_getlimit(char *class, char *msgpathbuf)
     struct aclmember *entry = NULL;
 
     if (msgpathbuf)
-        *msgpathbuf = NULL;
+        *msgpathbuf = '\0';
 
     /* limit <class> <n> <times> [<message_file>] */
     while (getaclentry("limit", &entry)) {
