@@ -80,6 +80,7 @@ extern char **ftpglob(register char *v),
  *globerr,
   remotehost[],
   hostname[],
+  fhostname[],
  *autospout,
   Shutdown[];
 
@@ -158,6 +159,7 @@ msg_massage(char *inbuf, char *outbuf)
     int limit;
     extern struct passwd *pw;
     struct aclmember *entry = NULL;
+    struct hostent *fhp;
 
     (void) time(&curtime);
     (void) acl_getclass(buffer);
@@ -204,7 +206,11 @@ msg_massage(char *inbuf, char *outbuf)
                 break;
 
             case 'L':
-                strcpy(outptr, hostname);
+                if (fhostname) {
+                    strcpy(outptr, fhostname);
+                } else {
+                    strcpy(outptr, hostname);
+                }
                 break;
 
             case 'U':
@@ -956,53 +962,39 @@ del_check(char *name)
   }
 }
 
-#define basename(x) (strrchr(x,'/')?1+strrchr(x,'/'):x)
-
-struct nr {
-	struct nr *next;
-	char *entry;
-};
-
-static struct nr *noretrieve_entry = NULL;
+#define lbasename(x) (strrchr(x,'/')?1+strrchr(x,'/'):x)
 
 int
 checknoretrieve (char *name)
 {
-  char cwd[MAXPATHLEN], realwd[MAXPATHLEN], realname[MAXPATHLEN];
+  char cwd[MAXPATHLEN+1], realwd[MAXPATHLEN+1], realname[MAXPATHLEN+1];
   int i;
   struct aclmember *entry = NULL;
-  struct nr *base;
 
-  if (!name || *name == (char) NULL)
+  if (name == (char *)NULL || *name == '\0')
     return 0;
 
-  (void) getwd (cwd);
+#ifdef HAVE_GETCWD
+  (void)getcwd (cwd, MAXPATHLEN);
+#else
+  (void)getwd (cwd);
+#endif
+
   realpath (cwd, realwd);
   realpath (name, realname);
 
-  if (noretrieve_entry == NULL)
-    {
-      base = noretrieve_entry = (struct nr *) malloc (sizeof (struct nr));
-      while (getaclentry ("noretrieve", &entry))
-	{
-	  if (!ARG0)
-	    continue;
-	  for (i = 0; i < MAXARGS && entry->arg[i] && *entry->arg[i]; i++)
-	    {
-	      base->entry = (char *) strdup (entry->arg[i]);
-	      base = base->next = (struct nr *) malloc (sizeof (struct nr));
-	      base->next = (struct nr *) NULL;
-	    }
-	}
-    }
-  for (base = noretrieve_entry; base; base = base->next)
-    {
-      if (!strcmp (*base->entry == '/' ? realname : basename (realname),
-		   base->entry))
-	{
-	  reply (550, "%s is marked unretrievable", base->entry);
-	  return 1;
-	}
-    }
-  return 0;
+   while (getaclentry("noretrieve", &entry)) {
+        if (ARG0 == (char *)NULL)
+            continue;
+        for (i = 0; i< MAXARGS &&
+             (entry->arg[i] != (char *)NULL) && (*(entry->arg[i]) !='\0'); i++)
+          if (strcmp (((*(entry->arg[i]) == '/') ? realname :
+                        lbasename (realname)), entry->arg[i]) == 0)
+          {
+            reply (550, "%s is marked unretrievable", entry->arg[i]);
+            return 1;
+          }
+      }
+   return 0;
 }
+
