@@ -158,10 +158,11 @@ acl_getlimit(char *aclbuf, char *class)
 
     while (*aclbuf != '\0') {
         if (strncasecmp(aclbuf, "limit", 5) == 0) {
-            for (crptr = aclbuf; *crptr++ != '\n';) ;
+            for (crptr = aclbuf; *crptr++ != '\n' && *crptr;) ;
             *--crptr = '\0';
             strcpy(linebuf, aclbuf);
             *crptr = '\n';
+	    printf(""); /* XXX - MvS - prevent coredump (??? Yes!) */
             (void) strtok(linebuf, " \t");  /* returns "limit" */
             if (strcmp(class, strtok(NULL, " \t")) == 0) {
                 limit = atoi(strtok(NULL, " \t"));  /* returns limit <n> */
