@@ -180,6 +180,7 @@ off_t byte_count;
 mode_t defumask = CMASK;           /* default umask value */
 char tmpline[7];
 char hostname[MAXHOSTNAMELEN];
+char fhostname[MAXHOSTNAMELEN];
 char remotehost[MAXHOSTNAMELEN];
 char remoteaddr[MAXHOSTNAMELEN];
 
@@ -260,6 +261,7 @@ main(int argc, char **argv, char **envp)
     int tos;
 #endif
     char *cp;
+    struct hostent *fhp;
 
 #ifdef SecureWare
     setluid(1);                         /* make sure there is a valid luid */
@@ -295,10 +297,12 @@ main(int argc, char **argv, char **envp)
     data_source.sin_port = htons(ntohs(ctrl_addr.sin_port) - 1);
     debug = 0;
 
+    tzset();                /* in case no timezone database in ~ftp */
+
 #ifdef FACILITY
-    openlog("ftpd", LOG_PID | LOG_NDELAY, FACILITY);
+    openlog("wu-ftpd", LOG_PID | LOG_NDELAY, FACILITY);
 #else
-    openlog("ftpd", LOG_PID);
+    openlog("wu-ftpd", LOG_PID);
 #endif
 
 #ifdef SETPROCTITLE
@@ -495,6 +499,11 @@ main(int argc, char **argv, char **envp)
     (void) gethostname(hostname, sizeof (hostname));
 #endif
 
+    if (fhp = gethostbyname(hostname)) {
+        strncpy(fhostname, fhp->h_name, sizeof(fhostname));
+        fhostname[sizeof(fhostname)-1]='\0';
+    }
+
     access_init();
     authenticate();
     conv_init();
@@ -994,8 +1003,9 @@ pass(char *passwd)
 #ifdef SHADOW_PASSWORD
         if (pw) {
            struct spwd *spw = getspnam( pw->pw_name );
-           if( !spw ) { pw->pw_passwd = ""; }
-           else { pw->pw_passwd = spw->sp_pwdp; }
+           if (spw)
+               pw->pw_passwd = spw->sp_pwdp;
+           endspent();
         }
 #endif
 
@@ -1020,8 +1030,23 @@ pass(char *passwd)
             reply(530, "Login incorrect.");
 
 #ifdef LOG_FAILED                       /* 27-Apr-93    EHK/BM             */
+
+	    /* log login failures to LOG_AUTH if possible */
+	    closelog();
+# ifdef FACILITY
+	    openlog("wu-ftpd", LOG_PID | LOG_ODELAY, LOG_AUTH);
+# else
+	    openlog("wu-ftpd", LOG_PID);
+# endif
             syslog(LOG_INFO, "failed login from %s [%s], %s",
                               remotehost, remoteaddr, the_user);
+	    closelog();
+# ifdef FACILITY
+	    openlog("wu-ftpd", LOG_PID | LOG_NDELAY, FACILITY);
+# else
+	    openlog("wu-ftpd", LOG_PID);
+# endif
+
 #endif
             acl_remove();
 
@@ -1815,19 +1840,19 @@ send_data(FILE *instr, FILE *outstr, off_t blksize)
         }
         netfd = fileno(outstr);
         filefd = fileno(instr);
-#ifdef __linux__
-	while ((cnt = read(filefd, buf, (u_int)blksize)) > 0)
-	{
-	int outcnt=0, newcnt=0;
-	while ((outcnt=write(netfd, buf+newcnt, cnt-newcnt))!= cnt-newcnt)
-		newcnt+=outcnt;
-	byte_count += cnt;				
+
+	while ((cnt = read(filefd, buf, (u_int)blksize)) > 0) {
+	    unsigned outcnt = cnt;
+	    char*newbuf = buf;
+	    do {
+	 	if ((cnt = write(netfd, newbuf, outcnt)) < 0)
+			goto writerr;
+		newbuf += cnt;
+		outcnt -= cnt;
+		byte_count += cnt;
+	    } while (outcnt);
 	}
-#else
-        while ((cnt = read(filefd, buf, (u_int) blksize)) > 0 &&
-               write(netfd, buf, cnt) == cnt)
-            byte_count += cnt;
-#endif
+writerr:
         transflag = 0;
         (void) free(buf);
         if (cnt != 0) {
@@ -2483,6 +2508,9 @@ passive(void)
     register char *p,
      *a;
 
+    if (pdata > 0)
+	close (pdata);
+
     pdata = socket(AF_INET, SOCK_STREAM, 0);
     if (pdata < 0) {
         perror_reply(425, "Can't open passive connection");
