@@ -29,10 +29,6 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifndef lint
-static char rcsid[] = "@(#)$Id: private.c,v 1.14 1999/02/26 16:08:50 sob RELEASE sob $";
-#endif /* not lint */
-
 #ifndef NO_PRIVATE
 
 #include "config.h"
@@ -46,6 +42,7 @@ static char rcsid[] = "@(#)$Id: private.c,v 1.14 1999/02/26 16:08:50 sob RELEASE
 #include <syslog.h>
 #endif
 #include <grp.h>
+#include <stdlib.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -77,26 +74,24 @@ extern char remotehost[],
 int group_attempts;
 
 void
-#ifdef __STDC__
 parsepriv(void)
-#else
-parsepriv()
-#endif
 {
     char *ptr;
     char *acptr = passbuf,
      *line;
-    char *argv[3],
+    char **argv[51],
+    **ap = (char **) argv,
      *p,
      *val;
-    struct acgrp *aptr,
-     *privtail = (struct acgrp *) NULL;
+    struct acgrp *aptr;
     struct group *gr;
-    int n;
 
     if (!passbuf || !(*passbuf))
         return;
 
+    /* build list, initialize to zero. */
+    privptr = (struct acgrp *) calloc(1, sizeof(struct acgrp));
+
     /* read through passbuf, stripping comments. */
     while (*acptr != '\0') {
         line = acptr;
@@ -111,35 +106,34 @@ parsepriv()
         if (*line == '\0')
             continue;
 
+        ap = (char **) argv;
+
         /* parse the lines... */
-        for (n = 0, p = line; n < 3 && p != NULL; n++) {
+        for (p = line; p != NULL;) {
             while ((val = (char *) strsep(&p, ":\n")) != NULL
-                   && *val == '\0'  && p != NULL)
-                ;
-            argv[n] = val;
-            if (argv[n][0] == ' ')
-                argv[n] = NULL;
+                   && *val == '\0') ;
+            *ap = val;
+            if (**ap == ' ')
+                *ap = NULL;
+            *ap++;
         }
-        /* check their were 3 fields, if not skip the line... */
-        if (n != 3 || p != NULL)
-            continue;
+        *ap = 0;
 
         if ((gr = getgrnam((char *) argv[2])) != NULL)  {
             aptr = (struct acgrp *) calloc(1, sizeof(struct acgrp));
             
-            /* add element to end of list */
-            if (privtail)
-                privtail->next = aptr;
-            privtail = aptr;
-            if (!privptr)
-                privptr = aptr;
+            /* if we didn't read 3 things, skip that line... */
+
+            /* add element to beginning of list */
+
+            aptr->next = privptr;
+            privptr = aptr;
 
             strcpy(aptr->gname, (char *) argv[0]);
             strcpy(aptr->gpass, (char *) argv[1]);
             strcpy(aptr->gr_name, (char *) argv[2]);
             aptr->gr_gid = gr->gr_gid;
         }
-	endgrent();
     }
 }
 
@@ -150,49 +144,40 @@ parsepriv()
 /*************************************************************************/
 
 void
-#ifdef __STDC__
 priv_setup(char *path)
-#else
-priv_setup(path)
-char *path;
-#endif
 {
     FILE *prvfile;
     struct stat finfo;
 
     passbuf = (char *) NULL;
 
+    if (stat(path, &finfo) != 0) {
+        syslog(LOG_ERR, "cannot stat private access file %s: %s", path,
+               strerror(errno));
+        return;
+    }
     if ((prvfile = fopen(path, "r")) == NULL) {
         if (errno != ENOENT)
             syslog(LOG_ERR, "cannot open private access file %s: %s",
                    path, strerror(errno));
         return;
     }
-    if (fstat(fileno(prvfile), &finfo) != 0) {
-        syslog(LOG_ERR, "cannot fstat private access file %s: %s", path,
-               strerror(errno));
-        (void) fclose(prvfile);
-        return;
-    }
     if (finfo.st_size == 0) {
         passbuf = (char *) calloc(1, 1);
     } else {
-        if (!(passbuf = (char *)malloc((unsigned) finfo.st_size + 1))) {
+        if (!(passbuf = malloc((unsigned) finfo.st_size + 1))) {
             (void) syslog(LOG_ERR, "could not malloc passbuf (%d bytes)",
                           finfo.st_size + 1);
-            (void) fclose(prvfile);
             return;
         }
         if (!fread(passbuf, (size_t) finfo.st_size, 1, prvfile)) {
             (void) syslog(LOG_ERR, "error reading private access file %s: %s",
                           path, strerror(errno));
-            (void) fclose(prvfile);
-            return;
         }
         *(passbuf + finfo.st_size) = '\0';
     }
-    (void) fclose(prvfile);
-    (void) parsepriv();
+
+	(void) parsepriv();
 }
 
 /*************************************************************************/
@@ -203,12 +188,7 @@ char *path;
 /*************************************************************************/
 
 struct acgrp *
-#ifdef __STDC__
 priv_getent(char *group)
-#else
-priv_getent(group)
-char *group;
-#endif
 {
 	struct acgrp *ptr;
 
@@ -226,14 +206,9 @@ char *group;
 /*************************************************************************/
 
 void
-#ifdef __STDC__
 priv_group(char *group)
-#else
-priv_group(group)
-char *group;
-#endif
 {
-    if ((int)strlen(group) < MAXGROUPLEN) {
+    if (strlen(group) < MAXGROUPLEN) {
         strncpy(groupname, group, MAXGROUPLEN);
         group_given = 1;
         reply(200, "Request for access to group %s accepted.", group);
@@ -252,25 +227,12 @@ char *group;
 /*************************************************************************/
 
 void
-#ifdef __STDC__
 priv_gpass(char *gpass)
-#else
-priv_gpass(gpass)
-char *gpass;
-#endif
 {
     char *xgpass,
      *salt;
 #ifndef NO_CRYPT_PROTO
-#ifdef __STDC__
     char *crypt(const char *, const char *);
-#else
-#ifdef _M_UNIX
-    char *crypt(char *, char *);
-#else
-    char *crypt();
-#endif
-#endif
 #endif
     struct acgrp *grp;
     struct group *gr;
@@ -289,7 +251,7 @@ char *gpass;
         grp = priv_getent(groupname);
 
         if (grp == NULL)
-            salt = "xx"; /* XXX */
+            salt = "xx";
         else
             salt = grp->gpass;
 
@@ -314,11 +276,9 @@ char *gpass;
     uid = geteuid();
     gid = grp->gr_gid;
 
-    delay_signaling(); /* we can't allow any signals while euid==0: kinch */
     seteuid(0);
     setegid(gid);
     seteuid(uid);
-    enable_signaling(); /* we can allow signals once again: kinch */
 
     reply(200, "Group access enabled.");
     group_attempts = 0;
