@@ -17,9 +17,9 @@
  *
  * @(#)utmp.h   5.10 (Berkeley) 6/1/90 */
 
-#define _PATH_UTMP  "/etc/utmp"
-#define _PATH_WTMP  "/usr/adm/wtmp"
-#define _PATH_LASTLOG   "/usr/adm/lastlog"
+#define _PATH_UTMP  "/var/run/utmp"
+#define _PATH_WTMP  "/var/log/wtmp"
+#define _PATH_LASTLOG   "/var/log/lastlog"
 
 #define UT_NAMESIZE 8
 #define UT_LINESIZE 8
